"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
/*
    Logic
*/
/**
 * Creates react context with state store. Store consists of state properties and actions which modify it.
 * @param initialState initial state
 *
 * @param actions a dictionary (key-value object) of action creators for a given store. Actions might have arguments, can modify state and return values.
 * Action creator as a first argument receives special [[ContextReference]] object, which has:
 * * `setState` method to modify state
 * * `state` object to access current state of type
 * * `stores` object to access injected store contexts
 *
 * Any other argument in action creator will be converted to action parameter, which user must provide when calling action.
 *
 * Example:
 * * Action creator:
 * ```typescript
 * actionName: (reference, payload: string) => {
 *      reference.setState({whatever: payload})
 * }
 * ```
 * * Action:
 * ```typescript
 * store.actionName("Hello world")
 * ```
 *
 * @param contexts a dictionary (key-value object) of injected store contexts. Those stores will be resolved and passed to actions in `stores` property of [[ContextReference]]
 *
 * Example:
 * ```typescript
 * createStoreContext({foo: '123'}, {
 *  action: (reference) => {
 *      reference.stores.bar.someBarAction()
 *      return reference.stores.bar.someBarProperty
 *  }
 * }, {
 *  bar: barContext
 * })
 * ```
 *
 * @typeparam TState any JS object representing application or application part state
 * @typeparam TActions a dictionary (key-value object) of action creators
 * @typeparam TContexts a dictionary (key-value object) of injected store contexts
 *
 * @returns [context, Provider] tuple, where context is [[React.Context]] and should be consumed to access store,
 * and Provider is [[React.FC]] (functional component) without props, which enables store actions for any nested components
 */
function createStoreContext(initialState, actions, contexts) {
    if (actions === void 0) { actions = {}; }
    if (contexts === void 0) { contexts = {}; }
    // tslint:disable-next-line: max-line-length
    var store = __assign({}, initialState, mapActionsToDefault(initialState, actions));
    var context = React.createContext(store);
    var provider = function (props) {
        var _a = React.useState(initialState), _state = _a[0], setState = _a[1];
        var stores = Object.keys(contexts).reduce(function (obj, key) {
            var _a;
            return __assign({}, obj, (_a = {}, _a[key] = React.useContext(contexts[key]), _a));
        }, {});
        var _actions = mapActionsToDispatch({
            state: _state,
            setState: setState,
            stores: stores
        }, actions);
        var _store = __assign({}, _state, _actions);
        return (React.createElement(context.Provider, { value: _store }, props.children));
    };
    return [context, provider];
}
exports.default = createStoreContext;
/**
 * @hidden internal function to map action creators to store actions
 */
function mapActionsToDispatch(contextReference, actions) {
    if (actions === undefined)
        return {};
    return Object.keys(actions).reduce(function (obj, key) {
        var _a;
        return __assign({}, obj, (_a = {}, _a[key] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return actions[key].apply(actions, [contextReference].concat(args));
        }, _a));
    }, {});
}
/**
 * @hidden internal function to map action creators to store actions when there are no Providers available
 */
function mapActionsToDefault(initialState, actions) {
    if (actions === undefined)
        return {};
    return Object.keys(actions).reduce(function (obj, key) {
        var _a;
        var contextReference = {
            state: initialState,
            stores: {},
            // tslint:disable-next-line: max-line-length
            setState: function (value) { throw new Error("[" + key + "]: Can't invoke 'setState' with " + value + " because provider does not exist"); }
        };
        return __assign({}, obj, (_a = {}, _a[key] = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return actions[key].apply(actions, [contextReference].concat(args));
        }, _a));
    }, {});
}
